<?php


define('FEEDS_NODE_UNPUBLISH_ORPHANED', 1);
define('FEEDS_NODE_DELETE_ORPHANED', 2);


class FeedsNodeProcessorGC extends FeedsNodeProcessor {
	
	/**
	* Implementation of FeedsProcessor::process().
	* 
	* This function has been copied from FeedsNodeProcessor, with a few changes to call 
  * garbage collection. (Changes are marked with comments labeled "GC".)
	*/
  public function process(FeedsImportBatch $batch, FeedsSource $source) {
		
		// GC: Keep track of all nids in this feed.
    $nids_in_feed = array();
    
    // Keep track of processed items in this pass, set total number of items.
    $processed = 0;
    if (!$batch->getTotal(FEEDS_PROCESSING)) {
      $batch->setTotal(FEEDS_PROCESSING, count($batch->items));
    }

    while ($item = $batch->shiftItem()) {
			
			// GC: If there's an existing nid, store that.
			$nid = $this->existingItemId($batch, $source);
      if ($nid > 0) {
        $nids_in_feed[] = $nid;
      }
			
      // Create/update if item does not exist or update existing is enabled.
      if (!($nid) || ($this->config['update_existing'] != FEEDS_SKIP_EXISTING)) {
        // Only proceed if item has actually changed.
        $hash = $this->hash($item);
        if (!empty($nid) && $hash == $this->getHash($nid)) {
          continue;
        }

        $node = $this->buildNode($nid, $source->feed_nid);
        $node->feeds_node_item->hash = $hash;

        // Map and save node. If errors occur don't stop but report them.
        try {
          $this->map($batch, $node);
          node_save($node);
          if (!empty($nid)) {
            $batch->updated++;
          }
          else {
            $batch->created++;
          }
        }
        catch (Exception $e) {
          drupal_set_message($e->getMessage(), 'warning');
          watchdog('feeds', $e->getMessage(), array(), WATCHDOG_WARNING);
        }
        
        // GC: Store any new nids created.
        if(!in_array($node->nid, $nids_in_feed)) {
          $nids_in_feed[] = $node->nid;
        }
      }

      $processed++;
      if ($processed >= variable_get('feeds_node_batch_size', FEEDS_NODE_BATCH_SIZE)) {
        $batch->setProgress(FEEDS_PROCESSING, $batch->created + $batch->updated);
        return;
      }
    }
    
    // Set messages.
    if ($batch->created) {
      drupal_set_message(format_plural($batch->created, 'Created @number @type node.', 'Created @number @type nodes.', array('@number' => $batch->created, '@type' => node_get_types('name', $this->config['content_type']))));
    }
    elseif ($batch->updated) {
      drupal_set_message(format_plural($batch->updated, 'Updated @number @type node.', 'Updated @number @type nodes.', array('@number' => $batch->updated, '@type' => node_get_types('name', $this->config['content_type']))));
    }
    else {
      drupal_set_message(t('There is no new content.'));
    }
    
    // GC: Call garbage collector.
    $garbage_counts = $this->garbage($source->feed_nid, $nids_in_feed);
    // GC: Output garbage colleciton messages
    if($garbage_counts['unpublished']) {
      drupal_set_message(t('Unpublished !number !type nodes.', array('!number' => $garbage_counts['unpublished'], '!type' => node_get_types('name', $this->config['content_type']))));
    }
    if($garbage_counts['published']) {
      drupal_set_message(t('Republished !number !type nodes.', array('!number' => $garbage_counts['published'], '!type' => node_get_types('name', $this->config['content_type']))));
    }
    if($garbage_counts['deleted']) {
      drupal_set_message(t('Deleted !number !type nodes.', array('!number' => $garbage_counts['deleted'], '!type' => node_get_types('name', $this->config['content_type']))));
    }
    
    $batch->setProgress(FEEDS_PROCESSING, FEEDS_BATCH_COMPLETE);
  }
  
  /**
   * Collects garbage
   * 
   * @param $nids_in_feed
   * 	These include all items that were in the feed, meaning:
   * 	1. New items that had just received their new nid
   * 	2. Existing items (= existing nodes)
   */
  public function garbage($feed_nid, $nids_in_feed) {
    $counts = array('unpublished' => 0, 'published' => 0, 'deleted' => 0);
    
    // Get all published items from this feed that are not in this feed.
    $sql = "SELECT n.nid FROM {feeds_node_item} fni LEFT JOIN {node} n ON fni.nid = n.nid WHERE fni.feed_nid = %d AND n.status = 1";
    if (!empty($nids_in_feed)) {
      $sql .= " AND n.nid NOT IN (" . db_placeholders($nids_in_feed, 'int') . ")";
    }
    $result = db_query($sql, array_merge(array($feed_nid), $nids_in_feed));
    
    while ($node = db_fetch_object($result)) {
      switch ($this->config['orphaned_action']) {
				case FEEDS_NODE_UNPUBLISH_ORPHANED:
					// Unpublish and save.
					$node = node_load(array('nid' => $node->nid));
					node_unpublish_action($node);
					node_save($node);
					$counts['unpublished']++;
					break;
				case FEEDS_NODE_DELETE_ORPHANED:
					// Delete.
					node_delete($node->nid);
					$counts['deleted']++;
					break;
			}
    }
    
    // Get all unpublished items that may be back in the feed.
    // Check for this only if the user chose to unpublish nodes in the first place.
    if (!empty($nids_in_feed) && $this->config['orphaned_action'] == FEEDS_NODE_UNPUBLISH_ORPHANED) {
			$sql = "SELECT n.nid FROM {feeds_node_item} fni LEFT JOIN {node} n ON fni.nid = n.nid WHERE fni.feed_nid = %d AND n.status = 0 AND n.nid IN (" . db_placeholders($nids_in_feed, 'int') . ")";
			$result = db_query($sql, array_merge(array($feed_nid), $nids_in_feed));
			
			while ($node = db_fetch_object($result)) {
				//Republish and save.
				$node = node_load(array('nid' => $node->nid));
				node_publish_action($node);
				node_save($node);
				$counts['published']++;
			}
    }
    
    return $counts;
  }
	
	/**
   * Override parent::configDefaults().
   */
  public function configDefaults() {
    $defaults = parent::configDefaults();
    return array_merge($defaults, array(
      'orphaned_action' => FEEDS_NODE_UNPUBLISH_ORPHANED,
    ));
  }
	
	/**
   * Override parent::configForm().
   */
  public function configForm(&$form_state) {
    $form = parent::configForm($form_state);
    $form['orphaned_action'] = array(
			'#type' => 'select',
			'#title' => t('Action for orphaned nodes'),
			'#description' => t('<em>Orphaned nodes</em> are nodes which were created for feed items that once existed in the feed, but are no longer there.<br>Choose which action to perform on these nodes. <strong>Note:</strong> Choosing <em>Unpublish</em> means that the feed will try to re-publish the orphaned node, in case its corresponding feed item reappears in the feed.'),
			'#options' => array(
				FEEDS_NODE_UNPUBLISH_ORPHANED => 'Unpublish orphaned nodes',
				FEEDS_NODE_DELETE_ORPHANED => 'Delete orphaned nodes'
			),
			'#default_value' => $this->config['orphaned_action'],
    );
    return $form;
  }
}
